=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-15 08:37:41 +0000
+++ bzrlib/branch.py	2005-09-15 08:43:31 +0000
@@ -583,6 +583,14 @@
             f.close()
 
 
+    def has_revision(self, revision_id):
+        """True if this branch has a copy of the revision.
+
+        This does not necessarily imply the revision is merge
+        or on the mainline."""
+        return revision_id in self.revision_store
+
+
     def get_revision_xml_file(self, revision_id):
         """Return XML file object for revision object."""
         if not revision_id or not isinstance(revision_id, basestring):
@@ -821,15 +829,15 @@
         from bzrlib.fetch import greedy_fetch
 
         greedy_fetch(to_branch=self, from_branch=other,
-                     revision_limit=stop_revision)
-
-        revision_ids = self.missing_revisions(other, stop_revision)
-
-        if len(revision_ids) > 0:
-            count = greedy_fetch(self, other, revision_ids[-1], pb)[0]
-        else:
-            count = 0
-        self.append_revision(*revision_ids)
+                     revision=stop_revision)
+
+        pullable_revs = self.missing_revisions(other, stop_revision)
+
+        if pullable_revs:
+            greedy_fetch(to_branch=self,
+                         from_branch=other,
+                         revision=pullable_revs[-1])
+            self.append_revision(*pullable_revs)
 
 
     def commit(self, *args, **kw):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2005-09-15 08:37:41 +0000
+++ bzrlib/fetch.py	2005-09-15 08:43:31 +0000
@@ -154,9 +154,11 @@
     def _copy_revisions(self, revs_to_fetch):
         i = 0
         for rev_id in revs_to_fetch:
+            i += 1
+            if self.to_branch.has_revision(rev_id):
+                continue
             self.pb.update('fetch revision', i, self.count_total)
             self._copy_one_revision(rev_id)
-            i += 1
             self.count_copied += 1
 
 
@@ -176,6 +178,7 @@
         parent_ids = [x.revision_id for x in rev.parents]
         self.to_weaves.add_text(INVENTORY_FILEID, rev_id,
                                             split_lines(inv_xml), parent_ids)
+
         self.to_branch.revision_store.add(StringIO(rev_xml), rev_id)
 
         

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-15 05:21:28 +0000
+++ bzrlib/selftest/__init__.py	2005-09-15 08:43:31 +0000
@@ -325,7 +325,7 @@
                    'bzrlib.selftest.testlog',
                    'bzrlib.selftest.testrevisionnamespaces',
                    'bzrlib.selftest.testbranch',
-#                   'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.testrevision',
 #                   'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
                    'bzrlib.selftest.testdiff',

